;;; **********************************************************************
;;; ** THOR Basic++                                                     **
;;; ** A free Basic for the Atari 8 Bit series               		**
;;; ** (c) 2015 THOR Software, Thomas Richter                           **
;;; ** This is based on Atari Basic, as developed by			**
;;; ** Paul Laughton and Kathleen O'Brien, Shepardson Microsystems	**
;;; ** as found in: The Atari BASIC SOURCE BOOK	by Bill Wilkinson	**
;;; ** Compute! Publications (1983)					**
;;; ** $Id: basicerror.asm,v 1.11 2015/09/06 08:11:44 thor Exp $	**
;;; **                                                                  **
;;; ** In this module:   Basic error codes and error handling 		**
;;; **********************************************************************

	.include "basicmacros.i"
	.include "basicerror.i"
	.include "basicio.i"
	.include "basicstatements.i"
	.include "basicexecution.i"
	.include "basicstack.i"
	.include "basicmain.i"
	.include "constants.i"
	.include "syntax.i"
	.include "mathpack.i"
	.include "editor.i"
	
	.segment "BasMain"
	
;;; *** CallError
;;; *** Retrieves the error number from the stack and stores
;;; *** it in the CurrentErrorNumber
	.global CallError
.proc	CallError
	pla
	sta PrintPtr
	pla
	sta PrintPtr+1
	ldy #1
	lda (PrintPtr),y
	sta CurrentErrorNumber
	;; runs into the following
.endproc
;;; *** Error	(Atari Basic ERROR)
;;; *** Generate the error code in CurrentErrorNumber
	.global Error
.proc	Error
	lda #0
	sta DirectFlag		;do not escape
	jsr Stop		;stop program execution
	lda TrapLine+1		;is TRAP active?
	bmi stopprogram
	;; here not: Execute the TRAP command
	ldx CurrentErrorNumber
	stx TrappedErrorNumber	;keep the last error number
	ldx #0
	stx CurrentErrorNumber		;reset the error number
	dex
	stx TrapLine+1		;turn off trapping
	;; target line number is now in A=hi,Y=lo
	ldy TrapLine
	jmp GotoLineAY		;execute the goto command, also restores the stack
	;; ok, no trapping. Print the error and stop the program
stopprogram:
	jsr PrintCR		;print a line feed
	lda #ErrorToken		;get the token for ERROR
	jsr PrintStatement
	lda CurrentErrorNumber
	ldx #0
	stx CurrentErrorNumber
	jsr PrintNumber
	;; runs into the following
.endproc
;;; *** AbortAtLineMessage (Atari Basic ERRM2)
;;; *** Prints the line number if in program mode,
;;; *** aborts program execution and returns to the
;;; *** syntax parser without printing READY
;;; *** This is also used by STOP
	.global AbortAtLineMessage
.proc	AbortAtLineMessage
	jsr TestIfDirect	;are we currently executing in direct mode?
	bmi done		;ok, nothing else
	lda #>AtLineMsg
	ldy #<AtLineMsg
	jsr PrintStringAY
	jsr PrintLineNumber
	;; since line offsets may change due to editing the
	;; source, invalidate absolute addresses
	inc GenerationCounter
done:
	jsr PrintCR		;another line feed
	jmp Syntax		;restart syntax parser, awaiting user input
.endproc

