;;; **********************************************************************
;;; ** THOR Basic++                                                     **
;;; ** A free Basic for the Atari 8 Bit series               		**
;;; ** (c) 2015 THOR Software, Thomas Richter                           **
;;; ** This is based on Atari Basic, as developed by			**
;;; ** Paul Laughton and Kathleen O'Brien, Shepardson Microsystems	**
;;; ** as found in: The Atari BASIC SOURCE BOOK	by Bill Wilkinson	**
;;; ** Compute! Publications (1983)					**
;;; ** $Id: operatorvector.asm,v 1.4 2015/09/05 20:28:47 thor Exp $	**
;;; **                                                                  **
;;; ** In this module:   The vector defining all operators		**
;;; **********************************************************************

	.include "basicmacros.i"
	.include "basicoperators.i"
	.include "syntax.i"
	
	.segment "BasMain"

OperatorCounter		.set	$1d
.macro	addVector token, location
	.word location-1
	.if token<>OperatorCounter
	.error "Operator token definition mismatch"
	.endif
OperatorCounter		.set OperatorCounter+1 
.endmacro

;;; The operator table. This stores the execution vector for each operator.
;;; It starts with the first "real" operator, the numeric "less or equal"
	.global OperatorVector
OperatorVector:			;(OPETAB in Atari Basic)
	addVector LessOrEqualToken,EvaluateLessOrEqual
	addVector NotEqualToken,EvaluateNotEqual
	addVector GreaterOrEqualToken,EvaluateGreaterOrEqual
	addVector LessThanToken,EvaluateLessThan
	addVector GreaterThanToken,EvaluateGreaterThan
	addVector EqualsToken,EvaluateEquals
	addVector PowerToken,EvaluatePower
	addVector MultiplyToken,EvaluateMultiply
	addVector PlusToken,EvaluatePlus
	addVector MinusToken,EvaluateMinus
	addVector DivideToken,EvaluateDivide
	addVector NotToken,EvaluateNot
	addVector OrToken,EvaluateOr
	addVector AndToken,EvaluateAnd
	addVector BracketOpenToken,EvaluateBracketOpen
	addVector BracketCloseToken,EvaluateBracketClose
	addVector NumericAssignmentToken,EvaluateNumericAssignment
	addVector StringAssignmentToken,EvaluateStringAssignment
	addVector StringLessOrEqualToken,EvaluateLessOrEqual 		;yes, these are all string
	addVector StringNotEqualToken,EvaluateNotEqual    		;comparisons, yet they use
	addVector StringGreaterOrEqualToken,EvaluateGreaterOrEqual 	;all the same entry points
	addVector StringLessThanToken,EvaluateLessThan		;as their numeric
	addVector StringGreaterThanToken,EvaluateGreaterThan		;counter-
	addVector StringEqualsToken,EvaluateEquals		;parts
	addVector UnaryPlusToken,EvaluateUnaryPlus
	addVector UnaryMinusToken,EvaluateUnaryMinus		
	addVector StringBracketOpenToken,EvaluateStringBracketOpen 	;substring operation
	addVector NumericArrayBracketOpenToken,EvaluateNumericArrayBracketOpen
	addVector DimNumericBracketOpenToken,EvaluateDimNumericBracketOpen
	addVector FunctionBracketOpenToken,EvaluateFunctionBracket
	addVector DimStringBracketOpen,EvaluateDimStringBracket
	addVector ArrayDimensionCommaToken,EvaluateArrayDimensionComma
	addVector STRToken,EvaluateSTR
	addVector CHRToken,EvaluateCHR
	addVector USRToken,EvaluateUSR
	addVector ASCToken,EvaluateASC
	addVector VALToken,EvaluateVAL
	addVector LENToken,EvaluateLEN
	addVector ADRToken,EvaluateADR
	addVector ATNToken,EvaluateATN
	addVector COSToken,EvaluateCOS
	addVector PEEKToken,EvaluatePeek
	addVector SINToken,EvaluateSIN
	addVector RNDToken,EvaluateRND
	addVector FREToken,EvaluateFRE
	addVector EXPToken,EvaluateEXP
	addVector LOGToken,EvaluateLOG
	addVector CLOGToken,EvaluateCLOG
	addVector SQRToken,EvaluateSQR
	addVector SGNToken,EvaluateSGN
	addVector ABSToken,EvaluateABS
	addVector INTToken,EvaluateINT
	addVector PADDLEToken,EvaluatePADDLE
	addVector STICKToken,EvaluateSTICK
	addVector PTRIGToken,EvaluatePTRIG
	addVector STRIGToken,EvaluateSTRIG

;;; *** The operator priority table, defines for each operator a left and right priority

.macro	definePriority left, right
	.byte (left<<4)|right
.endmacro

	;; RevA had a bug evaluating not not X. In rev B this was "fixed" by changing the parser
	;; and disallowing the parse of not not. This is the wrong fix! The expression makes
	;; sense (not not converts to boolean). Here we fix this by assigning not and the unary + and -
	;; a lower right priority, i.e. association binds stronger to the right so that the right "not"
	;; is evaluated before the left "not".
	.global OperatorPriorityTable ;OPRTAB in Atari Basic
OperatorPriorityTable:	
	definePriority	0,0		;double quote
	definePriority	0,0		;end of expression
	definePriority	0,0		;comma
	definePriority	0,0		;$
	definePriority	0,0		;colon
	definePriority	0,0		;semicolon
	definePriority	0,0		;end of line
	definePriority	0,0		;on x goto
	definePriority	0,0		;on x gosub
	definePriority	0,0		;for x TO
	definePriority	0,0		;STEP
	definePriority	0,0		;THEN
	definePriority	0,0		;IOCB
	;; numeric comparisons have the lowest priority
	definePriority	7,7		;<=
	definePriority 	7,7		;<>
	definePriority	7,7		;>=
	definePriority	7,7		;>	(huh, why is the priority different?)
	definePriority	7,7		;<
	definePriority	7,7		;=
	definePriority	10,10		;^
	definePriority	9,9		;*
	definePriority	8,8		;+
	definePriority	8,8		;-
	definePriority	9,9		;/
	definePriority	13,12		;NOT	;rev.a had a lower priority here, rev b and c bind as strongly as +-.
	definePriority	5,5		;OR
	definePriority	6,6		;AND
	definePriority	15,2		;(
	definePriority	4,14		;)
	definePriority	15,1		;numeric assignment
	definePriority	15,1		;string assignment
	definePriority	14,14		;<= on strings
	definePriority	14,14		;<> on strings
	definePriority	14,14		;>= on strings
	definePriority	14,14		;< on strings
	definePriority	14,14		;> on strings
	definePriority	14,14		;= on strings
	definePriority	13,12		;+ (unary)
	definePriority	13,12		;- (unary)
	definePriority	15,2		;( for substrings
	definePriority	15,2		;( for arrays
	definePriority	15,2		;( for dimensioning arrays
	definePriority	15,2		;( on function calls
	definePriority	15,2		;( on string dimension
	definePriority	4,3		;, in array dimensions and to separate function call arguments
	;; functions follow. They all bind weakly to the left (i.e. first evaluate the contents, then themselves
	definePriority	15,2		;STR$
	definePriority	15,2		;CHR$
	definePriority	15,2		;USR
	definePriority	15,2		;ASC
	definePriority	15,2		;VAL
	definePriority	15,2		;LEN
	definePriority	15,2		;ADR
	definePriority	15,2		;ATN
	definePriority	15,2		;COS
	definePriority	15,2		;PEEK
	definePriority	15,2		;SIN
	definePriority	15,2		;RND
	definePriority	15,2		;FRE
	definePriority	15,2		;EXP
	definePriority	15,2		;LOG
	definePriority	15,2		;CLOG
	definePriority	15,2		;SQR
	definePriority	15,2		;SGN
	definePriority	15,2		;ABS
	definePriority	15,2		;INT
	definePriority	15,2		;PADDLE
	definePriority	15,2		;STICK
	definePriority	15,2		;PTRIG
	definePriority	15,2		;STRIG
	
